import { Space, Table, Button, Input, Select, Divider, Tag, Card, Badge, Form } from 'antd';
import type { TableColumnsType } from 'antd';
import { t } from '@/utils/i18n';
import React, { useState, useEffect } from 'react';
import { PlusOutlined, ExclamationCircleFilled, SearchOutlined, UndoOutlined } from '@ant-design/icons';
import type { Menu } from '@/models'
import { antdUtils } from '@/utils/antd';
import { useRequest } from '@/hooks/use-request';
import menuService from '@/request/service/menu';
import { formatDate } from '@/utils/formatTime'
import MenuEditor from './menu-editor';

const MenuListPage: React.FC = () => {

    const [menuEditorVisable, seMenuEditorVisable] = useState<boolean>(false);
    const [editMenu, seEditMenu] = useState<Menu>();
    const [dataSource, setDataSource] = useState<Menu[]>([]);
    const [searchFrom] = Form.useForm();


    const { runAsync: getMenuList } = useRequest(menuService.getMenuList, { manual: true });
    const { runAsync: deleteMenu } = useRequest(menuService.deleteMenu, { manual: true });

    const loadMenus = async () => {
        const [error, { data: menus }] = await getMenuList(searchFrom.getFieldsValue());
        if (!error) {
            const menuIds = menus.map(it => it.id)
            const rootMenus = menus.filter((menu) => menuIds.indexOf(menu.parentId ?? 0) < 0)
            menus.forEach(menu => {
                const parentMenu = menus.find(it => it.id === menu.parentId); //menu.parentId
                if (parentMenu) {
                    if (!parentMenu.children) {
                        parentMenu.children = []
                    }
                    parentMenu.children.push(menu)
                }
            })
            setDataSource(rootMenus);
        }
    };


    const showDeleteConfirm = (menu: Menu) => {
        antdUtils.modal?.confirm({
            title: '确认要将该菜单删除吗？',
            icon: <ExclamationCircleFilled />,
            content: '请注意删除以后不可恢复！',
            okText: '删除',
            okType: 'danger',
            cancelText: '取消',
            onOk() {
                return new Promise(async (resolve) => {
                    const [error, { code, msg} ] = await deleteMenu({ id: menu.id });
                    if(error || code !== 0) {
                        antdUtils.message?.open({ type: 'error', content: msg??'操作失败'})
                    } else {
                        antdUtils.message?.open({ type: 'success', content: '删除成功'})
                    }
                    await loadMenus();
                    resolve('')
                }).catch(() => antdUtils.message?.open({
                    type: 'error',
                    content: '操作失败',
                }));
            },
            onCancel() {
            },
        });
    };

    const columns: TableColumnsType<Menu> = [
        {
            title: '菜单名称',
            dataIndex: 'name',
            key: 'name',
            align: 'right',
            width: 200,
        },
        {
            title: '菜单类型',
            dataIndex: 'type',
            key: 'type',
            align: 'right',
            width: 100,
            render: (value: number, record, index) => {
                return (value === 1 ? <Tag color="purple">目录</Tag> : (value === 2 ? <Tag color="blue">菜单</Tag> : <Tag color="green">按钮</Tag>))
            }
        },
        {
            title: '路由地址',
            dataIndex: 'path',
            key: 'path',
            width: 150
        },
        {
            title: '组件路径',
            dataIndex: 'component',
            key: 'component',
            width: 150
        },
        {
            title: '权限标识',
            key: 'permission',
            dataIndex: 'permission',
        },
        {
            title: '排序',
            key: 'sort',
            dataIndex: 'sort',
            width: 100,
        },
        {
            title: '显示状态',
            key: 'visible',
            dataIndex: 'visible',
            width: 100,
            render: (value: boolean) => {
                return (value ? <Badge status="success" text="已显示" /> : <Badge status="warning" text="已隐藏" />)
            }
        },
        {
            title: '开启状态',
            key: 'status',
            dataIndex: 'status',
            width: 100,
            render: (value: number) => {
                return (value === 0 ? <Badge status="success" text="已开启" /> : <Badge status="error" text="已关闭" />)
            }
        },
        {
            title: '创建时间',
            key: 'createTime',
            dataIndex: 'createTime',
            width: 200,
            render: (value: number) => {
                return formatDate(new Date(value), "YYYY-mm-dd HH:MM:SS")
            }
        },
        {
            title: t("QkOmYwne" /* 操作 */),
            key: 'action',
            render: (menu: Menu, record) => (
                <Space size="small" split={( <Divider type='vertical' />)}>
                    <a onClick={() => {
                            seEditMenu(menu);
                            seMenuEditorVisable(true);
                        }}>
                        编辑
                    </a>
                    <a onClick={() => {
                            showDeleteConfirm(menu)
                        }}>
                        删除
                    </a>
                </Space>
            ),
        },
    ];

    useEffect(() => {
        loadMenus();
    }, []);

    const onReset = () => {
        searchFrom.resetFields()
        loadMenus()
    }

    return (
        <>
            <div>
                <Card className='mt-[4px] dark:bg-[rgb(33,41,70)] bg-white roundle-lg px[12px]' bodyStyle={{ paddingTop: 4, paddingBottom: 4 }}>
                    <div className='flex justify-between content-center'>
                        <div className='flex justify-normal items-center'>
                            <Form layout='inline' form={searchFrom}>
                                <Form.Item name="name" label="菜单名称">
                                    <Input placeholder='请输入名称' allowClear/>
                                </Form.Item>
                                <Form.Item className='ml-2 w-[130px]' name="status" label="状态">
                                    <Select placeholder="请选择" allowClear >
                                        <Select.Option value="2">全部</Select.Option>
                                        <Select.Option value="0">开启</Select.Option>
                                        <Select.Option value="1">关闭</Select.Option>
                                    </Select>
                                </Form.Item>
                            </Form>
                            <Space.Compact className="ml-5">
                                <Button type='primary' size='large' icon={<SearchOutlined />} onClick={loadMenus}> 搜索 </Button>
                                <Button type='primary' size='large' icon={<UndoOutlined />} onClick={onReset}> 重置 </Button>
                            </Space.Compact>
                        </div>
                        <div className="py-[4px]">
                            <Button className="ml-5" type='primary' size='large' icon={<PlusOutlined />} onClick={() => {
                            seEditMenu(undefined);
                            seMenuEditorVisable(true);
                        }}> 添加菜单 </Button>
                        </div>
                    </div>
                </Card>
                <Card className='mt-[4px] dark:bg-[rgb(33,41,70)] bg-white roundle-lg px[12px]'>
                    <Table rowKey="id"
                        scroll={{ x: true }}
                        columns={columns}
                        dataSource={dataSource}
                        className='bg-transparent'
                        pagination={{ position: ['bottomRight'] }} />
                </Card>
            </div>
            <MenuEditor
                parentMenus={dataSource}
                onSave={() => { 
                    loadMenus();
                    seMenuEditorVisable(false);
                }}
                onCancel={() => { seMenuEditorVisable(false) }}
                visible={menuEditorVisable}
                editData={editMenu} />
        </>
    );
};

export default MenuListPage;
